﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel;
using System.Diagnostics;
using System.Management.Automation; 

namespace PSModuleSample
{

    [Cmdlet(VerbsCommon.Get, "Status",DefaultParameterSetName="ByName")]
    public class GetStatusCommand : Cmdlet
    {

        protected override void ProcessRecord()
        {
            if (Status != ComputerStatus.Default)
            {
                var servers = ServerList.Instance.Servers.Where(x => x.Status == Status);
                foreach (var server in servers)
                {
                    WriteObject(server, true);
                }
            }
            else if (Name != null)
            {
                var servers = ServerList.Instance.Servers.Where(x =>
                {
                    return Name.Any(y => new WildcardPattern(y, WildcardOptions.IgnoreCase).IsMatch(x.Name));
                });

                foreach(var server in servers)
                {
                    WriteObject(server, true);
                }
            }
            else
            {
                ServerList.Instance.Servers.ForEach(x => WriteObject(x, true));
            }
        }

        [Parameter(Mandatory = false, ValueFromPipeline = false, Position = 0,ParameterSetName = "ByName")]
        public string[] Name { get; set; }

        [Parameter(Mandatory = true, ValueFromPipeline = false, ParameterSetName = "ByStatus")]
        public ComputerStatus Status { get; set; }
    }


    [Cmdlet(VerbsCommon.Set, "Status", SupportsShouldProcess = true)]
    public class SetStatusCommand : Cmdlet
    {
        protected override void ProcessRecord()
        {
            if (Name != null)
            {
                var server = ServerList.Instance.Servers.Where(x => x.Name == Name).FirstOrDefault();
                if (server == null)
                {
                    throw new Exception("指定のサーバーが存在しません");
                }
                else
                {
                    if (ShouldProcess(Name, string.Format("ステータスを{0}に変更", Status)))
                    {
                        server.Status = Status;
                        if (PassThru) 
                        {
                            WriteObject(server, true);
                        }
                    }
                }
            }
            else if (InputObject != null)
            {
                foreach (var server in InputObject)
                {
                    if (ShouldProcess(server.Name, string.Format("ステータスを{0}に変更", Status)))
                    {
                        server.Status = Status;
                        if (PassThru)
                        {
                            WriteObject(server, true);
                        }
                    }
                }
            }
        }

        [Parameter(Mandatory=true, ValueFromPipeline = false, Position = 0, ParameterSetName = "NameValue")]
        public string Name { get; set; }

        [Parameter(Mandatory = true, ValueFromPipeline = false, Position = 1)]
        public ComputerStatus Status { get; set; }

        [Parameter(Mandatory = false, ValueFromPipeline = false, Position = 10)]
        public SwitchParameter PassThru { get; set; }

        [Parameter(Mandatory = true, ValueFromPipeline = true, Position = 0, ParameterSetName = "InputObject")]
        public Server[] InputObject { get; set; }
    }

    public class Server
    {
        public string Name { get; set; }
        public ComputerStatus Status { get; set; }
        public DateTime StartDate { get; set; }
        public Server(string name, ComputerStatus status, DateTime startDate)
        {
            Name = name;
            Status = status;
            StartDate = startDate;
        }
    }

    public enum ComputerStatus
    {
        Default,Good,Warning,Error
    }

    public sealed class ServerList
    {
        public static readonly ServerList Instance = new ServerList();
        private ServerList()
        {
            _servers = new List<Server>() { 
                new Server("FileServer1", ComputerStatus.Good, new DateTime(2012,10,10,5,42,11)),
                new Server("WebServer",ComputerStatus.Good,  new DateTime(2013,1,2,12,11,43)),
                new Server("DomainController",ComputerStatus.Warning,  new DateTime(2013,2,1,0,0,0)) ,
                new Server("DHCPServer",ComputerStatus.Good,new DateTime(2012,12,12,12,12,12)),
                new Server("FileServer2",ComputerStatus.Good,new DateTime(2012,11,30,22,59,17))
            };
        }

        private List<Server> _servers;

        public List<Server> Servers
        {
            get { return _servers; }
            set { _servers = value; }
        }
    }
}
